@@ -36,7 +36,8 @@
  * if it could cause failure when a lookup fails, but that's hard.
  */
 static Odr_oid *zoom_yaz_str_to_z3950oid(ZOOM_connection c,
-                                     oid_class oid_class, const char *str) {
+                                     oid_class oid_class, const char *str)
+{
     Odr_oid *res = yaz_string_to_oid_odr(yaz_oid_std(), oid_class, str,
                                      c->odr_out);
     if (res == 0)
@@ -465,7 +466,7 @@ static Z_APDU *create_update_package(ZOOM_package p)
 static void otherInfo_attach(ZOOM_connection c, Z_APDU *a, ODR out)
 {
     int i;
-    for (i = 0; i<200; i++)
+    for (i = 0; i < 200; i++)
     {
         size_t len;
         Odr_oid *oid;
@@ -633,28 +634,28 @@ zoom_ret ZOOM_connection_Z3950_send_init(ZOOM_connection c)
     return send_APDU(c, apdu);
 }
 
-zoom_ret ZOOM_connection_Z3950_send_search(ZOOM_connection c)
+static zoom_ret Z3950_send_search(ZOOM_connection c)
 {
     ZOOM_resultset r;
     int lslb, ssub, mspn;
     const char *syntax;
     const char *schema;
-    Z_APDU *apdu = zget_APDU(c->odr_out, Z_APDU_searchRequest);
-    Z_SearchRequest *search_req = apdu->u.searchRequest;
+    Z_APDU *apdu;
+    Z_SearchRequest *search_req;
     const char *elementSetName;
     const char *smallSetElementSetName;
     const char *mediumSetElementSetName;
-    const char *facets;
 
     assert(c->tasks);
     assert(c->tasks->which == ZOOM_TASK_SEARCH);
-
     r = c->tasks->u.search.resultset;
 
-    yaz_log(c->log_details, "%p ZOOM_connection_send_search set=%p", c, r);
+    apdu = zget_APDU(c->odr_out, Z_APDU_searchRequest);
+    search_req = apdu->u.searchRequest;
 
-    elementSetName =
-        ZOOM_options_get(r->options, "elementSetName");
+    yaz_log(c->log_details, "%p Z3950_send_search set=%p", c, r);
+
+    elementSetName = c->tasks->u.search.elementSetName;
     smallSetElementSetName  =
         ZOOM_options_get(r->options, "smallSetElementSetName");
     mediumSetElementSetName =
@@ -666,15 +667,17 @@ zoom_ret ZOOM_connection_Z3950_send_search(ZOOM_connection c)
     if (!mediumSetElementSetName)
         mediumSetElementSetName = elementSetName;
 
-    facets = ZOOM_options_get(r->options, "facets");
-    if (facets) {
-        Z_FacetList *facet_list = yaz_pqf_parse_facet_list(c->odr_out, facets);
-        if (facet_list) {
+    if (r->req_facets)
+    {
+        Z_FacetList *facet_list =
+            yaz_pqf_parse_facet_list(c->odr_out, r->req_facets);
+        if (facet_list)
+        {
             Z_OtherInformation **oi = &search_req->additionalSearchInfo;
             yaz_oi_set_facetlist(oi, c->odr_out, facet_list);
         }
         else
-            yaz_log(YLOG_WARN, "Unable to parse facets: %s", facets);
+            yaz_log(YLOG_WARN, "Unable to parse facets: %s", r->req_facets);
     }
 
     assert(r);
@@ -786,7 +789,7 @@ zoom_ret ZOOM_connection_Z3950_send_search(ZOOM_connection c)
                 if (!rp)
                     break;
             }
-            r->setname = xstrdup(setname);
+            r->setname = odr_strdup(r->odr, setname);
             yaz_log(c->log_details, "%p ZOOM_connection_send_search: "
                     "allocating set %s", c, r->setname);
         }
@@ -794,7 +797,7 @@ zoom_ret ZOOM_connection_Z3950_send_search(ZOOM_connection c)
         {
             yaz_log(c->log_details, "%p ZOOM_connection_send_search: using "
                     "default set", c);
-            r->setname = xstrdup("default");
+            r->setname = odr_strdup(r->odr, "default");
         }
         ZOOM_options_set(r->options, "setname", r->setname);
     }
@@ -984,16 +987,21 @@ static int es_response_taskpackage_update(ZOOM_connection c,
     if (utp && utp->targetPart)
     {
         Z_IUTargetPart *targetPart = utp->targetPart;
-        switch ( *targetPart->updateStatus ) {
+        switch (*targetPart->updateStatus)
+        {
         case Z_IUTargetPart_success:
-            ZOOM_options_set(c->tasks->u.package->options,"updateStatus", "success");
+            ZOOM_options_set(c->tasks->u.package->options, 
+                             "updateStatus", "success");
             break;
         case Z_IUTargetPart_partial:
-            ZOOM_options_set(c->tasks->u.package->options,"updateStatus", "partial");
+            ZOOM_options_set(c->tasks->u.package->options,
+                             "updateStatus", "partial");
             break;
         case Z_IUTargetPart_failure:
-            ZOOM_options_set(c->tasks->u.package->options,"updateStatus", "failure");
-            if (targetPart->globalDiagnostics && targetPart->num_globalDiagnostics > 0)
+            ZOOM_options_set(c->tasks->u.package->options,
+                             "updateStatus", "failure");
+            if (targetPart->globalDiagnostics &&
+                targetPart->num_globalDiagnostics > 0)
                 response_diag(c, targetPart->globalDiagnostics[0]);
             break;
         }
@@ -1009,8 +1017,8 @@ static int es_response_taskpackage(ZOOM_connection c,
     if (id)
         ZOOM_options_setl(c->tasks->u.package->options,
                           "targetReference", (char*) id->buf, id->len);
-
-    switch ( *taskPackage->taskStatus ) {
+    switch (*taskPackage->taskStatus)
+    {
     case Z_TaskPackage_pending:
         ZOOM_options_set(c->tasks->u.package->options,"taskStatus", "pending");
         break;
@@ -1022,25 +1030,27 @@ static int es_response_taskpackage(ZOOM_connection c,
         break;
     case Z_TaskPackage_aborted:
         ZOOM_options_set(c->tasks->u.package->options,"taskStatus", "aborted");
-        if ( taskPackage->num_packageDiagnostics && taskPackage->packageDiagnostics )
+        if (taskPackage->num_packageDiagnostics &&
+            taskPackage->packageDiagnostics )
             response_diag(c, taskPackage->packageDiagnostics[0]);
         break;
     }
     /* NOTE: Only Update implemented, no others. */
-    if ( taskPackage->taskSpecificParameters->which == Z_External_update )
+    if (taskPackage->taskSpecificParameters->which == Z_External_update)
     {
-        Z_IUUpdateTaskPackage *utp = taskPackage->taskSpecificParameters->u.update->u.taskPackage;
+        Z_IUUpdateTaskPackage *utp =
+            taskPackage->taskSpecificParameters->u.update->u.taskPackage;
         es_response_taskpackage_update(c, utp);
     }
     return 1;
 }
 
-
-static int handle_Z3950_es_response(ZOOM_connection c,
-                                    Z_ExtendedServicesResponse *res)
+static void handle_Z3950_es_response(ZOOM_connection c,
+                                     Z_ExtendedServicesResponse *res)
 {
-    if (!c->tasks || c->tasks->which != ZOOM_TASK_PACKAGE)
-        return 0;
+    if (!c->tasks)
+        return;
+    assert(c->tasks->which == ZOOM_TASK_PACKAGE);
     switch (*res->operationStatus)
     {
     case Z_ExtendedServicesResponse_done:
@@ -1068,14 +1078,14 @@ static int handle_Z3950_es_response(ZOOM_connection c,
         ZOOM_options_setl(c->tasks->u.package->options,
                           "xmlUpdateDoc", (char*) doc->buf, doc->len);
     }
-    return 1;
 }
 
-static char *get_term_cstr(ODR odr, Z_Term *term) {
-
-    switch (term->which) {
+static char *get_term_cstr(ODR odr, Z_Term *term)
+{
+    switch (term->which)
+    {
     case Z_Term_general:
-            return odr_strdupn(odr, (const char *) term->u.general->buf, (size_t) term->u.general->len);
+        return odr_strdupn(odr, term->u.general->buf, term->u.general->len);
         break;
     case Z_Term_characterString:
         return odr_strdup(odr, term->u.characterString);
@@ -1083,22 +1093,27 @@ static char *get_term_cstr(ODR odr, Z_Term *term) {
     return 0;
 }
 
-static ZOOM_facet_field get_zoom_facet_field(ODR odr, Z_FacetField *facet) {
-    int term_index;
+static ZOOM_facet_field get_zoom_facet_field(ODR odr, Z_FacetField *facet)
+{
+    int i;
     struct yaz_facet_attr attr_values;
     ZOOM_facet_field facet_field = odr_malloc(odr, sizeof(*facet_field));
     yaz_facet_attr_init(&attr_values);
     yaz_facet_attr_get_z_attributes(facet->attributes, &attr_values);
     facet_field->facet_name = odr_strdup(odr, attr_values.useattr);
     facet_field->num_terms = facet->num_terms;
-    yaz_log(YLOG_DEBUG, "ZOOM_facet_field %s %d terms %d", attr_values.useattr, attr_values.limit, facet->num_terms);
-    facet_field->facet_terms = odr_malloc(odr, facet_field->num_terms * sizeof(*facet_field->facet_terms));
-    for (term_index = 0 ; term_index < facet->num_terms; term_index++) {
-        Z_FacetTerm *facetTerm = facet->terms[term_index];
-        facet_field->facet_terms[term_index].frequency = *facetTerm->count;
-        facet_field->facet_terms[term_index].term = get_term_cstr(odr, facetTerm->term);
+    yaz_log(YLOG_DEBUG, "ZOOM_facet_field %s %d terms %d",
+            attr_values.useattr, attr_values.limit, facet->num_terms);
+    facet_field->facet_terms =
+        odr_malloc(odr, facet->num_terms * sizeof(*facet_field->facet_terms));
+    for (i = 0 ; i < facet->num_terms; i++)
+    {
+        Z_FacetTerm *facetTerm = facet->terms[i];
+        facet_field->facet_terms[i].frequency = *facetTerm->count;
+        facet_field->facet_terms[i].term = get_term_cstr(odr, facetTerm->term);
         yaz_log(YLOG_DEBUG, "    term[%d] %s %d",
-                term_index, facet_field->facet_terms[term_index].term, facet_field->facet_terms[term_index].frequency);
+                i, facet_field->facet_terms[i].term,
+                facet_field->facet_terms[i].frequency);
     }
     return facet_field;
 }
@@ -1107,21 +1122,26 @@ static ZOOM_facet_field get_zoom_facet_field(ODR odr, Z_FacetField *facet) {
 void ZOOM_handle_facet_list(ZOOM_resultset r, Z_FacetList *fl)
 {
     int j;
-    r->num_facets   = fl->num;
+    r->num_res_facets = fl->num;
     yaz_log(YLOG_DEBUG, "Facets found: %d", fl->num);
-    r->facets       =  odr_malloc(r->odr, r->num_facets * sizeof(*r->facets));
-    r->facets_names =  odr_malloc(r->odr, r->num_facets * sizeof(*r->facets_names));
+    r->res_facets =  odr_malloc(r->odr, fl->num * sizeof(*r->res_facets));
+    r->facets_names =  odr_malloc(r->odr, fl->num * sizeof(*r->facets_names));
     for (j = 0; j < fl->num; j++)
     {
-        r->facets[j] = get_zoom_facet_field(r->odr, fl->elements[j]);
-        if (!r->facets[j])
+        r->res_facets[j] = get_zoom_facet_field(r->odr, fl->elements[j]);
+        if (!r->res_facets[j])
+        {
+            r->facets_names[j] = 0;
             yaz_log(YLOG_DEBUG, "Facet field missing on index %d !", j);
-        r->facets_names[j] = (char *) ZOOM_facet_field_name(r->facets[j]);
+        }
+        else
+            r->facets_names[j] = (char *)
+                ZOOM_facet_field_name(r->res_facets[j]);
     }
 }
 
-static void handle_facet_result(ZOOM_connection c, ZOOM_resultset r,
-                                Z_OtherInformation *o)
+void ZOOM_handle_facet_result(ZOOM_connection c, ZOOM_resultset r,
+                              Z_OtherInformation *o)
 {
     int i;
     for (i = 0; o && i < o->num_elements; i++)
@@ -1144,8 +1164,7 @@ static void handle_queryExpressionTerm(ZOOM_options opt, const char *name,
     {
     case Z_Term_general:
         ZOOM_options_setl(opt, name,
-                          (const char *)(term->u.general->buf),
-                          term->u.general->len);
+                          term->u.general->buf, term->u.general->len);
         break;
     case Z_Term_characterString:
         ZOOM_options_set(opt, name, term->u.characterString);
@@ -1176,8 +1195,8 @@ static void handle_queryExpression(ZOOM_options opt, const char *name,
 }
 
 
-static void handle_search_result(ZOOM_connection c, ZOOM_resultset resultset,
-                                Z_OtherInformation *o)
+void ZOOM_handle_search_result(ZOOM_connection c, ZOOM_resultset resultset,
+                               Z_OtherInformation *o)
 {
     int i;
     for (i = 0; o && i < o->num_elements; i++)
@@ -1249,17 +1268,22 @@ static void handle_Z3950_search_response(ZOOM_connection c,
 {
     ZOOM_resultset resultset;
     ZOOM_Event event;
+    const char *resultCountPrecision = "exact";
 
     if (!c->tasks || c->tasks->which != ZOOM_TASK_SEARCH)
-        return ;
-
-    event = ZOOM_Event_create(ZOOM_EVENT_RECV_SEARCH);
-    ZOOM_connection_put_event(c, event);
+        return;
 
     resultset = c->tasks->u.search.resultset;
 
+    if (resultset->live_set == 0)
+    {
+        event = ZOOM_Event_create(ZOOM_EVENT_RECV_SEARCH);
+        ZOOM_connection_put_event(c, event);
+    }
     if (sr->resultSetStatus)
     {
+        if (*sr->resultSetStatus == Z_SearchResponse_estimate)
+            resultCountPrecision = "estimate";
         ZOOM_options_set_int(resultset->options, "resultSetStatus",
                              *sr->resultSetStatus);
     }
@@ -1268,11 +1292,17 @@ static void handle_Z3950_search_response(ZOOM_connection c,
         ZOOM_options_set_int(resultset->options, "presentStatus",
                              *sr->presentStatus);
     }
-    handle_search_result(c, resultset, sr->additionalSearchInfo);
+    ZOOM_options_set(resultset->options, "resultCountPrecision",
+                     resultCountPrecision);
+    ZOOM_handle_search_result(c, resultset, sr->additionalSearchInfo);
 
-    handle_facet_result(c, resultset, sr->additionalSearchInfo);
+    ZOOM_handle_facet_result(c, resultset, sr->additionalSearchInfo);
 
     resultset->size = *sr->resultCount;
+
+    ZOOM_memcached_hitcount(c, resultset, sr->additionalSearchInfo,
+                            resultCountPrecision);
+    resultset->live_set = 2;
     handle_Z3950_records(c, sr->records, 0);
 }
 
@@ -1282,13 +1312,13 @@ static void handle_Z3950_sort_response(ZOOM_connection c, Z_SortResponse *res)
         response_diag(c, res->diagnostics[0]);
 }
 
-static int handle_Z3950_scan_response(ZOOM_connection c, Z_ScanResponse *res)
+static void handle_Z3950_scan_response(ZOOM_connection c, Z_ScanResponse *res)
 {
     NMEM nmem = odr_extract_mem(c->odr_in);
     ZOOM_scanset scan;
 
     if (!c->tasks || c->tasks->which != ZOOM_TASK_SCAN)
-        return 0;
+        return;
     scan = c->tasks->u.scan.scan;
 
     if (res->entries && res->entries->nonsurrogateDiagnostics)
@@ -1306,7 +1336,6 @@ static int handle_Z3950_scan_response(ZOOM_connection c, Z_ScanResponse *res)
         ZOOM_options_set_int(scan->options, "number",
                              *res->numberOfEntriesReturned);
     nmem_destroy(nmem);
-    return 1;
 }
 
 static void handle_Z3950_records(ZOOM_connection c, Z_Records *sr,
@@ -1316,29 +1345,16 @@ static void handle_Z3950_records(ZOOM_connection c, Z_Records *sr,
     int *start, *count;
     const char *syntax = 0, *elementSetName = 0, *schema = 0;
 
-    if (!c->tasks)
+    if (!c->tasks || c->tasks->which != ZOOM_TASK_SEARCH)
         return ;
-    switch (c->tasks->which)
-    {
-    case ZOOM_TASK_SEARCH:
-        resultset = c->tasks->u.search.resultset;
-        start = &c->tasks->u.search.start;
-        count = &c->tasks->u.search.count;
-        syntax = c->tasks->u.search.syntax;
-        elementSetName = c->tasks->u.search.elementSetName;
-        schema =  c->tasks->u.search.schema;
-        break;
-    case ZOOM_TASK_RETRIEVE:
-        resultset = c->tasks->u.retrieve.resultset;
-        start = &c->tasks->u.retrieve.start;
-        count = &c->tasks->u.retrieve.count;
-        syntax = c->tasks->u.retrieve.syntax;
-        elementSetName = c->tasks->u.retrieve.elementSetName;
-        schema =  c->tasks->u.retrieve.schema;
-        break;
-    default:
-        return;
-    }
+
+    resultset = c->tasks->u.search.resultset;
+    start = &c->tasks->u.search.start;
+    count = &c->tasks->u.search.count;
+    syntax = c->tasks->u.search.syntax;
+    elementSetName = c->tasks->u.search.elementSetName;
+    schema =  c->tasks->u.search.schema;
+
     if (sr && sr->which == Z_Records_NSD)
         response_default_diag(c, sr->u.nonSurrogateDiagnostic);
     else if (sr && sr->which == Z_Records_multipleNSD)
@@ -1360,7 +1376,7 @@ static void handle_Z3950_records(ZOOM_connection c, Z_Records *sr,
             NMEM nmem = odr_extract_mem(c->odr_in);
             Z_NamePlusRecordList *p =
                 sr->u.databaseOrSurDiagnostics;
-            for (i = 0; i<p->num_records; i++)
+            for (i = 0; i < p->num_records; i++)
             {
                 ZOOM_record_cache_add(resultset, p->records[i], i + *start,
                                       syntax, elementSetName, schema, 0);
@@ -1419,7 +1435,6 @@ static void set_init_option(const char *name, void *clientData)
     ZOOM_connection_option_set(c, buf, "1");
 }
 
-
 zoom_ret send_Z3950_sort(ZOOM_connection c, ZOOM_resultset resultset)
 {
     if (c->error)
@@ -1429,6 +1444,8 @@ zoom_ret send_Z3950_sort(ZOOM_connection c, ZOOM_resultset resultset)
         Z_APDU *apdu = zget_APDU(c->odr_out, Z_APDU_sortRequest);
         Z_SortRequest *req = apdu->u.sortRequest;
 
+        yaz_log(c->log_details, "%p send_Z3950_sort set=%p", c, resultset);
+
         req->num_inputResultSetNames = 1;
         req->inputResultSetNames = (Z_InternationalString **)
             odr_malloc(c->odr_out, sizeof(*req->inputResultSetNames));
@@ -1442,94 +1459,26 @@ zoom_ret send_Z3950_sort(ZOOM_connection c, ZOOM_resultset resultset)
     return zoom_complete;
 }
 
-zoom_ret send_Z3950_present(ZOOM_connection c)
+static zoom_ret Z3950_send_present(ZOOM_connection c)
 {
-    Z_APDU *apdu = 0;
-    Z_PresentRequest *req = 0;
-    int i = 0;
-    const char *syntax = 0;
-    const char *elementSetName = 0;
-    const char *schema = 0;
-    ZOOM_resultset resultset;
-    int *start, *count;
+    Z_APDU *apdu = zget_APDU(c->odr_out, Z_APDU_presentRequest);
+    Z_PresentRequest *req = apdu->u.presentRequest;
+    ZOOM_resultset resultset = c->tasks->u.search.resultset;
+    const char *syntax = c->tasks->u.search.syntax;
+    const char *elementSetName = c->tasks->u.search.elementSetName;
+    const char *schema = c->tasks->u.search.schema;
 
-    if (!c->tasks)
-    {
-        yaz_log(c->log_details, "%p send_present no tasks", c);
-        return zoom_complete;
-    }
+    yaz_log(c->log_details, "%p Z3950_send_present", c);
 
-    switch (c->tasks->which)
-    {
-    case ZOOM_TASK_SEARCH:
-        resultset = c->tasks->u.search.resultset;
-        start = &c->tasks->u.search.start;
-        count = &c->tasks->u.search.count;
-        syntax = c->tasks->u.search.syntax;
-        elementSetName = c->tasks->u.search.elementSetName;
-        schema =  c->tasks->u.search.schema;
-        break;
-    case ZOOM_TASK_RETRIEVE:
-        resultset = c->tasks->u.retrieve.resultset;
-        start = &c->tasks->u.retrieve.start;
-        count = &c->tasks->u.retrieve.count;
-        syntax = c->tasks->u.retrieve.syntax;
-        elementSetName = c->tasks->u.retrieve.elementSetName;
-        schema = c->tasks->u.retrieve.schema;
-        break;
-    default:
-        return zoom_complete;
-    }
-    yaz_log(c->log_details, "%p send_present start=%d count=%d",
-            c, *start, *count);
+    *req->resultSetStartPoint = c->tasks->u.search.start + 1;
 
-    if (*start < 0 || *count < 0 || *start + *count > resultset->size)
-    {
-        ZOOM_set_dset_error(c, YAZ_BIB1_PRESENT_REQUEST_OUT_OF_RANGE, "Bib-1",
-                       "", 0);
-    }
-    if (c->error)                  /* don't continue on error */
-        return zoom_complete;
-    yaz_log(c->log_details, "send_present resultset=%p start=%d count=%d",
-            resultset, *start, *count);
-
-    for (i = 0; i < *count; i++)
-    {
-        ZOOM_record rec =
-            ZOOM_record_cache_lookup(resultset, i + *start,
-                                     syntax, elementSetName, schema);
-        if (!rec)
-            break;
-        else
-        {
-            ZOOM_Event event = ZOOM_Event_create(ZOOM_EVENT_RECV_RECORD);
-            ZOOM_connection_put_event(c, event);
-        }
-    }
-    *start += i;
-    *count -= i;
-
-    if (*count == 0)
-    {
-        yaz_log(c->log_details, "%p send_present skip=%d no more to fetch", c, i);
-        return zoom_complete;
-    }
-
-    apdu = zget_APDU(c->odr_out, Z_APDU_presentRequest);
-    req = apdu->u.presentRequest;
-
-    if (i)
-        yaz_log(c->log_details, "%p send_present skip=%d", c, i);
-
-    *req->resultSetStartPoint = *start + 1;
-
-    if (resultset->step > 0 && resultset->step < *count)
+    if (resultset->step > 0 && resultset->step < c->tasks->u.search.count)
         *req->numberOfRecordsRequested = resultset->step;
     else
-        *req->numberOfRecordsRequested = *count;
+        *req->numberOfRecordsRequested = c->tasks->u.search.count;
 
-    if (*req->numberOfRecordsRequested + *start > resultset->size)
-        *req->numberOfRecordsRequested = resultset->size - *start;
+    if (*req->numberOfRecordsRequested + c->tasks->u.search.start > resultset->size)
+        *req->numberOfRecordsRequested = resultset->size - c->tasks->u.search.start;
     assert(*req->numberOfRecordsRequested > 0);
 
     if (syntax && *syntax)
@@ -1596,6 +1545,67 @@ zoom_ret send_Z3950_present(ZOOM_connection c)
     return send_APDU(c, apdu);
 }
 
+zoom_ret ZOOM_connection_Z3950_search(ZOOM_connection c)
+{
+    int i = 0;
+    const char *syntax = 0;
+    const char *elementSetName = 0;
+    const char *schema = 0;
+    ZOOM_resultset resultset;
+    int *start, *count;
+
+    if (!c->tasks)
+        return zoom_complete;
+    assert(c->tasks->which == ZOOM_TASK_SEARCH);
+    resultset = c->tasks->u.search.resultset;
+    start = &c->tasks->u.search.start;
+    count = &c->tasks->u.search.count;
+    syntax = c->tasks->u.search.syntax;
+    elementSetName = c->tasks->u.search.elementSetName;
+    schema =  c->tasks->u.search.schema;
+
+    yaz_log(c->log_details, "%p ZOOM_connection_Z3950_search start=%d count=%d",
+            c, *start, *count);
+
+    ZOOM_memcached_search(c, resultset);
+
+    if (*start < 0 || *count < 0)
+    {
+        ZOOM_set_dset_error(c, YAZ_BIB1_PRESENT_REQUEST_OUT_OF_RANGE, "Bib-1",
+                       "start/count < 0", 0);
+    }
+
+    if (resultset->live_set)
+    {
+        if (*start >= resultset->size)
+            return zoom_complete;
+        if (*start + *count > resultset->size)
+            *count = resultset->size - *start;
+    }
+
+    if (c->error)                  /* don't continue on error */
+        return zoom_complete;
+
+    for (i = 0; i < *count; i++)
+    {
+        ZOOM_record rec =
+            ZOOM_record_cache_lookup(resultset, i + *start,
+                                     syntax, elementSetName, schema);
+        if (!rec)
+            break;
+    }
+    *start += i;
+    *count -= i;
+
+    if (*count == 0 && resultset->live_set)
+        return zoom_complete;
+
+    if (resultset->live_set == 2)
+        return Z3950_send_present(c);
+    else
+        return Z3950_send_search(c);
+}
+
 static zoom_ret send_Z3950_sort_present(ZOOM_connection c)
 {
     zoom_ret r = zoom_complete;
@@ -1603,7 +1613,7 @@ static zoom_ret send_Z3950_sort_present(ZOOM_connection c)
     if (c->tasks && c->tasks->which == ZOOM_TASK_SEARCH)
         r = send_Z3950_sort(c, c->tasks->u.search.resultset);
     if (r == zoom_complete)
-        r = send_Z3950_present(c);
+        r = ZOOM_connection_Z3950_search(c);
     return r;
 }
 
@@ -1708,13 +1718,13 @@ void ZOOM_handle_Z3950_apdu(ZOOM_connection c, Z_APDU *apdu)
     case Z_APDU_presentResponse:
         yaz_log(c->log_api, "%p handle_Z3950_apdu Present response", c);
         handle_Z3950_present_response(c, apdu->u.presentResponse);
-        if (send_Z3950_present(c) == zoom_complete)
+        if (ZOOM_connection_Z3950_search(c) == zoom_complete)
             ZOOM_connection_remove_task(c);
         break;
     case Z_APDU_sortResponse:
         yaz_log(c->log_api, "%p handle_Z3950_apdu Sort response", c);
         handle_Z3950_sort_response(c, apdu->u.sortResponse);
-        if (send_Z3950_present(c) == zoom_complete)
+        if (ZOOM_connection_Z3950_search(c) == zoom_complete)
             ZOOM_connection_remove_task(c);
         break;
     case Z_APDU_scanResponse:
